@@ -1,6 +1,6 @@
 /* channel.c
  *
- * $Id: channel.c,v 1.111 2002/05/06 06:51:15 nisse Exp $
+ * $Id: channel.c,v 1.113 2002/06/27 17:10:01 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -640,7 +640,7 @@ DEFINE_PACKET_HANDLER(static, global_request_handler, connection, packet)
 	  else
 	    {
 	      /* We should ignore failures. */
-	      static struct report_exception_info global_req_ignore =
+	      static const struct report_exception_info global_req_ignore =
 		STATIC_REPORT_EXCEPTION_INFO(EXC_ALL, EXC_GLOBAL_REQUEST,
 					     "Ignored:");
 	      
@@ -880,7 +880,8 @@ DEFINE_PACKET_HANDLER(static, channel_request_handler,
 	      else
 		{
 		  /* We should ignore failures. */
-		  static struct report_exception_info channel_req_ignore =
+		  static const struct report_exception_info
+		    channel_req_ignore =
 		    STATIC_REPORT_EXCEPTION_INFO(EXC_ALL, EXC_CHANNEL_REQUEST,
 						 "Ignored:");
 		  
@@ -2001,7 +2002,8 @@ make_channel_read_close_callback(struct ssh_channel *channel)
      (super exception_handler)
      (vars
        (channel object ssh_channel)
-       (prefix . "const char *")))
+       (prefix . "const char *")
+       (silent . int)))
 */
 
 static void
@@ -2011,7 +2013,8 @@ do_channel_io_exception_handler(struct exception_handler *s,
   CAST(channel_io_exception_handler, self, s);
   if (x->type & EXC_IO)
     {
-      werror("channel.c: I/O error on write, %z\n", x->msg);
+      if (!self->silent)
+	werror("channel.c: %zI/O error, %z\n", self->prefix, x->msg);
 #if 0
       send_debug_message(self->channel->write,
 			 ssh_format("%z I/O error: %z\n",
@@ -2027,6 +2030,7 @@ do_channel_io_exception_handler(struct exception_handler *s,
 struct exception_handler *
 make_channel_io_exception_handler(struct ssh_channel *channel,
 				  const char *prefix,
+				  int silent,
 				  struct exception_handler *parent,
 				  const char *context)
 {
@@ -2037,7 +2041,7 @@ make_channel_io_exception_handler(struct ssh_channel *channel,
   
   self->channel = channel;
   self->prefix = prefix;
-
+  self->silent = silent;
   return &self->super;
 }
 
