@@ -1,6 +1,6 @@
 /* client_session.c
  *
- * $Id: client_session.c,v 1.18 2002/05/02 20:52:32 nisse Exp $
+ * $Id: client_session.c,v 1.19 2002/06/26 15:56:25 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -141,13 +141,13 @@ do_client_io(struct command *s UNUSED,
    * start. */
   session->out->e
     = make_channel_io_exception_handler(channel,
-					"lsh: I/O error on stdout",
+					"stdout: ", 0,
 					session->out->e,
 					HANDLER_CONTEXT);
 
   session->err->e
     = make_channel_io_exception_handler(channel,
-					"lsh: I/O error on stderr",
+					"stderr: ", 0,
 					session->err->e,
 					HANDLER_CONTEXT);
 
