@@ -1,6 +1,6 @@
 /* command.h
  *
- * $Id: command.h,v 1.59 2001/07/06 21:11:01 nisse Exp $ */
+ * $Id: command.h,v 1.60 2002/06/27 17:17:31 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -291,11 +291,11 @@ do_catch_simple(struct command *s,
      (name catch_report_collect)
      (super command)
      (vars
-       (info object report_exception_info)))
+       (info const object report_exception_info)))
 */
 
 struct command *
-make_catch_report_apply(struct report_exception_info *info,
+make_catch_report_apply(const struct report_exception_info *info,
 			struct command *body);
 
 void
