@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: crypto.h,v 1.29 2001/10/15 12:40:38 nisse Exp $ */
+ * $Id: crypto.h,v 1.31 2002/01/20 21:16:24 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -43,10 +43,11 @@ extern struct crypto_algorithm crypto_des3_cbc_algorithm;
 extern struct crypto_algorithm crypto_serpent256_cbc_algorithm;
 extern struct crypto_algorithm crypto_twofish256_cbc_algorithm;
 
-extern struct hash_algorithm sha1_algorithm;
-extern struct hash_algorithm md5_algorithm;
+extern const struct hash_algorithm crypto_sha1_algorithm;
+extern const struct hash_algorithm crypto_md5_algorithm;
 
-struct mac_algorithm *make_hmac_algorithm(struct hash_algorithm *h);
+struct mac_algorithm *
+make_hmac_algorithm(const struct hash_algorithm *h);
 
 /* 10 million iterations would take 5 hours on my machine */
 #define PKCS5_MAX_ITERATIONS 10000000
