@@ -2,7 +2,7 @@
  *
  *
  *
- * $Id: debug.c,v 1.35 2001/04/04 21:50:48 nisse Exp $ */
+ * $Id: debug.c,v 1.36 2002/03/13 15:40:37 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -91,11 +91,14 @@ make_debug_packet(const char *msg, int always_display)
 
 /* Send a debug message to the other end. */
 void
-send_debug_message(struct abstract_write *write, const char *msg, int always_display)
+send_debug_message(struct ssh_connection *connection,
+		   const char *msg, int always_display)
 {
-  A_WRITE(write, make_debug_packet(msg, always_display));
+  /* Can be sent even during key exchange. */
+  C_WRITE_NOW(connection, make_debug_packet(msg, always_display));
 }
 
+#if 0
 void
 send_debug(struct abstract_write *write, const char *msg, int always_display)
 {
@@ -109,6 +112,7 @@ send_verbose(struct abstract_write *write, const char *msg, int always_display)
   if (verbose_flag)
     send_debug_message(write, msg, always_display);
 }
+#endif
 
 DEFINE_PACKET_HANDLER(, connection_debug_handler, connection UNUSED, packet)
 {
