@@ -1,6 +1,6 @@
 /* io.c
  *
- * $Id: io.c,v 1.166 2002/06/02 15:45:45 nisse Exp $ */
+ * $Id: io.c,v 1.168 2002/06/26 16:00:40 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -649,14 +649,22 @@ do_write_callback(struct io_callback *s UNUSED,
 	    break;
 	  case EPIPE:
 	    debug("io.c: Broken pipe.\n");
-	
+
 	    /* Fall through */
 	  default:
+#if 0
+	    /* Don't complain if it was writing the final ^D
+	     * character that failed. */
+	    if ( (fd->type == IO_PTY_CLOSED)
+		 && (fd->write_buffer->length == 1) )
+	      debug("io.c: ignoring write error, on the final ^D character\n");
+#endif
 	    werror("io.c: write failed, %z\n", STRERROR(errno));
 	    EXCEPTION_RAISE(fd->e,
-			    make_io_exception(EXC_IO_WRITE, fd, errno, NULL));
+			    make_io_exception(EXC_IO_WRITE,
+					      fd, errno, NULL));
 	    close_fd(fd);
-	
+	    
 	    break;
 	  }
       else
@@ -1324,6 +1332,7 @@ make_lsh_fd(int fd, const char *label,
 
   self->fd = fd;
   self->label = label;
+  self->type = 0;
   
   self->e = make_exception_handler(do_exc_io_handler, e, HANDLER_CONTEXT);
   
@@ -1339,6 +1348,14 @@ make_lsh_fd(int fd, const char *label,
   return self;
 }
 
+void
+io_set_type(struct lsh_fd *fd, enum io_type type)
+{
+  trace("io_set_type: fd %i, type %i\n",
+	fd->fd, type);
+  fd->type = type;
+}
+
 unsigned
 io_nfiles(void)
 {
@@ -1756,7 +1773,7 @@ close_fd(struct lsh_fd *fd)
        * buffer fails. */
       if (fd->write_buffer)
 	fd->write_buffer->closed = 1;
-  
+      
       if (fd->close_callback)
 	LSH_CALLBACK(fd->close_callback);
       
@@ -1784,21 +1801,17 @@ close_fd_nicely(struct lsh_fd *fd)
 
   trace("io.c: close_fd_nicely called on fd %i: %z\n",
 	fd->fd, fd->label);
-  
+
+  /* Stop reading */
   fd->read = NULL;
-
   lsh_oop_cancel_read_fd(fd);
-  
-  if (fd->write_buffer)
-    {
-      /* Mark the write_buffer as closed */
-      fd->write_buffer->closed = 1;
-      if (!fd->write_buffer->empty)
-	return;
-    }
 
-  /* There's no data buffered for write. */
-  close_fd(fd);
+  if (fd->write_buffer)
+    /* Close after currently buffered data is written out. */
+    close_fd_write(fd);
+  else
+    /* There's no data buffered for write. */
+    close_fd(fd);
 }
 
 /* Stop reading, but if the fd has a write callback, keep it open. */
@@ -1822,6 +1835,19 @@ close_fd_write(struct lsh_fd *fd)
     {
       /* Mark the write_buffer as closed */
       fd->write_buffer->closed = 1;
+
+      if (fd->type == IO_PTY)
+	{
+	  debug("Writing ^D to pty.\n");
+	  /* Is there any better way to signal EOF on a pty? This is
+	   * what emacs does. */
+	  A_WRITE(&fd->write_buffer->super,
+		  ssh_format("%lc", /* C-d */ 4));
+
+	  /* No need to repeat this. */
+	  fd->type = 0;
+	}
+      
       if (fd->write_buffer->empty)
 	{
           if (!fd->read)
