@@ -2,7 +2,7 @@
  *
  * main proxy program.
  *
- * $Id: lsh_proxy.c,v 1.31 2002/03/15 09:12:37 nisse Exp $ */
+ * $Id: lsh_proxy.c,v 1.32 2002/06/27 17:19:22 nisse Exp $ */
 
 /* lsh, an implementation of the ssh protocol
  *
@@ -572,6 +572,7 @@ int main(int argc, char **argv)
   /* FIXME: Choose character set depending on the locale */
   set_local_charset(CHARSET_LATIN1);
 
+  /* FIXME: Use STATIC_REPORT_EXCEPTION_INFO */
   handler = make_report_exception_handler
     (make_report_exception_info(EXC_IO, EXC_IO, "lsh_proxy: "),
      &default_exception_handler,
@@ -689,7 +690,7 @@ int main(int argc, char **argv)
 #if WITH_X11_FORWARD
     if (options->with_x11_forward)
       {
-	ALIST_SET(server_session_requests, ATOM_X11_REQ, &gateway_channel_request);
+	ALIST_SET(server_session_requests, ATOM_X11_REQ, &gateway_channel_request.super);
       }
 #endif
 #if WITH_AGENT_FORWARD
