@@ -1,11 +1,11 @@
 /* server_session.c
  *
- * $Id: server_session.c,v 1.84 2002/05/29 22:50:51 _cvs_pont Exp $
+ * $Id: server_session.c,v 1.92 2002/07/05 20:03:02 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
  *
- * Copyright (C) 1998 Niels Mller
+ * Copyright (C) 1998, 2002 Niels Mller
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -29,6 +29,7 @@
 #include "read_data.h"
 #include "reaper.h"
 #include "server_pty.h"
+#include "server_x11.h"
 #include "ssh.h"
 #include "tcpforward.h"
 #include "translate_signal.h"
@@ -52,6 +53,9 @@
 
 #include "server_session.c.x"
 
+/* Disabled in the 1.4 series. */
+#undef WITH_X11_FORWARD
+#define WITH_X11_FORWARD 0
 
 /* Session */
 /* GABA:
@@ -67,8 +71,12 @@
        ; Resource to kill when the channel is closed. 
        (process object lsh_process)
 
-       ; pty
+       ; An allocated but not yet used pty
        (pty object pty_info)
+
+       ; X11 forwarding.
+       (x11 object server_x11_info)
+       
        ; value of the TERM environment variable
        (term string)
 
@@ -132,15 +140,7 @@ do_eof(struct ssh_channel *channel)
   CAST(server_session, session, channel);
 
   trace("server_session.c: do_eof\n");
-
-  if (session->pty)
-    /* Is there any better way to signal EOF on a pty? This is what
-     * emacs does. */
-    /* FIXME: This should be handled specially by close_fd_write, so
-     * that we can ignore EPIPE errors. */
-    A_WRITE(&session->in->write_buffer->super,
-            ssh_format("%lc", /* C-d */ 4));
-
+  
   close_fd_write(session->in);
 }
 
@@ -166,6 +166,9 @@ make_server_session(UINT32 initial_window,
    * is taken care of automatically. */
   
   self->process = NULL;
+
+  self->pty = NULL;
+  self->term = NULL;
   
   self->in = NULL;
   self->out = NULL;
@@ -418,11 +421,10 @@ static int make_pty(struct pty_info *pty UNUSED,
 static int
 spawn_process(struct server_session *session,
 	      struct lsh_user *user,
-	      /* All information but the fd:s should be fileld in
+	      /* All information but the fd:s should be filled in
 	       * already */
 	      struct spawn_info *info)
 {
-  struct exception_handler *io_exception_handler;
   struct lsh_callback *read_close_callback;
   
   assert(!session->process);
@@ -443,22 +445,22 @@ spawn_process(struct server_session *session,
   if (!session->process)
     return 0;
   
-  /* Exception handlers */
-  io_exception_handler
-    = make_channel_io_exception_handler(&session->super,
-					"lshd: Child stdio: ",
-					&default_exception_handler,
-					HANDLER_CONTEXT);
-
   /* Close callback for stderr and stdout */
   read_close_callback
     = make_channel_read_close_callback(&session->super);
   
   session->in
-    = io_write(make_lsh_fd(info->in[1], "child stdin",
-			   io_exception_handler),
+    = io_write(make_lsh_fd
+	       (info->in[1], "child stdin",
+		make_channel_io_exception_handler(&session->super,
+						  "Child stdin: ", 0,
+						  &default_exception_handler,
+						  HANDLER_CONTEXT)),
 	       SSH_MAX_PACKET, NULL);
-	  
+
+  if (session->pty)
+    io_set_type(session->in, IO_PTY);
+  
   /* Flow control */
   session->in->write_buffer->report = &session->super.super;
   
@@ -466,14 +468,22 @@ spawn_process(struct server_session *session,
    * which will close the channel on read errors, or is it
    * better to just send EOF on read errors? */
   session->out
-    = io_read(make_lsh_fd(info->out[0], "child stdout",
-			  io_exception_handler),
+    = io_read(make_lsh_fd
+	      (info->out[0], "child stdout",
+	       make_channel_io_exception_handler(&session->super,
+						 "Child stdout: ", 1,
+						 &default_exception_handler,
+						 HANDLER_CONTEXT)),
 	      make_channel_read_data(&session->super),
 	      read_close_callback);
   session->err 
     = ( (info->err[0] != -1)
-	? io_read(make_lsh_fd(info->err[0], "child stderr",
-			      io_exception_handler),
+	? io_read(make_lsh_fd
+		  (info->err[0], "child stderr",
+		   make_channel_io_exception_handler(&session->super,
+						     "Child stderr: ", 0,
+						     &default_exception_handler,
+						     HANDLER_CONTEXT)),
 		  make_channel_read_stderr(&session->super),
 		  read_close_callback)
 	: NULL);
@@ -516,9 +526,9 @@ init_spawn_info(struct spawn_info *info, struct server_session *session,
   info->argc = argc;
   info->argv = argv;
   
-  assert(env_length >= 3);
+  assert(env_length >= 5);
 
-  /* FIXME: Set SSH_TTY, SSH_CLIENT and SSH_ORIGINAL_COMMAND */
+  /* FIXME: Set SSH_ORIGINAL_COMMAND */
   if (session->term)
     {
       env[i].name ="TERM";
@@ -551,6 +561,18 @@ init_spawn_info(struct spawn_info *info, struct server_session *session,
       i++;
     }
 
+#if WITH_X11_FORWARD
+  if (session->x11)
+    {
+      env[i].name = "DISPLAY";
+      env[i].value = session->x11->display;
+      i++;
+
+      env[i].name = "XAUTHORITY";
+      env[i].value = session->x11->xauthority;
+      i++;
+    }
+#endif /* WITH_X11_FORWARD */
   assert(i <= env_length);
   info->env_length = i;
   info->env = env;
@@ -566,7 +588,7 @@ DEFINE_CHANNEL_REQUEST(shell_request_handler)
 {
   CAST(server_session, session, channel);
   struct spawn_info spawn;
-  struct env_value env[3];
+  struct env_value env[5];
   
   static const struct exception shell_request_failed =
     STATIC_EXCEPTION(EXC_CHANNEL_REQUEST, "Shell request failed");
@@ -581,7 +603,7 @@ DEFINE_CHANNEL_REQUEST(shell_request_handler)
     /* Already spawned a shell or command */
     goto fail;
 
-  init_spawn_info(&spawn, session, 0, NULL, 3, env);
+  init_spawn_info(&spawn, session, 0, NULL, 5, env);
   spawn.login = 1;
     
   if (spawn_process(session, channel->connection->user, &spawn))
@@ -627,11 +649,11 @@ DEFINE_CHANNEL_REQUEST(exec_request_handler)
     {
       struct spawn_info spawn;
       const char *args[2];
-      struct env_value env[3];
+      struct env_value env[5];
 
       struct lsh_string *s = ssh_format("%ls", command_len, command);
       
-      init_spawn_info(&spawn, session, 2, args, 3, env);
+      init_spawn_info(&spawn, session, 2, args, 5, env);
       spawn.login = 0;
       
       args[0] = "-c";
@@ -716,7 +738,7 @@ do_spawn_subsystem(struct channel_request *s,
     {
       struct spawn_info spawn;
       const char *args[2];
-      struct env_value env[3];
+      struct env_value env[5];
 
       /* Don't use any pty */
       if (session->pty)
@@ -727,7 +749,7 @@ do_spawn_subsystem(struct channel_request *s,
 
       args[0] = "-c"; args[1] = program;
       
-      init_spawn_info(&spawn, session, 2, args, 3, env);
+      init_spawn_info(&spawn, session, 2, args, 5, env);
       spawn.login = 0;
 
       if (spawn_process(session, channel->connection->user, &spawn))
@@ -858,3 +880,65 @@ window_change_request_handler =
 { STATIC_HEADER, do_window_change_request };
 
 #endif /* WITH_PTY_SUPPORT */
+
+#if WITH_X11_FORWARD
+
+/* FIXME: We must delay the handling of any shell request until
+ * we have responded to the x11-req message. */
+static void
+do_x11_req(struct channel_request *s UNUSED,
+	   struct ssh_channel *channel,
+	   struct channel_request_info *info UNUSED,
+	   struct simple_buffer *args,
+	   struct command_continuation *c,
+	   struct exception_handler *e)
+{
+  static struct exception x11_request_failed =
+    STATIC_EXCEPTION(EXC_CHANNEL_REQUEST, "x11-req failed");
+
+  const UINT8 *protocol;
+  UINT32 protocol_length;
+  const UINT8 *cookie;
+  UINT32 cookie_length;
+  UINT32 screen;
+  unsigned single;
+  
+  CAST(server_session, session, channel);
+
+  verbose("Client requesting x11 forwarding...\n");
+
+  if (parse_uint8(args, &single)
+      && parse_string(args, &protocol_length, &protocol)
+      && parse_string(args, &cookie_length, &cookie)
+      && parse_uint32(args, &screen))
+    {
+      /* The client may only request one x11-forwarding, and only
+       * before starting a process. */
+      if (session->x11 || session->process
+	  || !(session->x11 = server_x11_setup(channel,
+					       channel->connection->user,
+					       protocol_length, protocol,
+					       cookie_length, cookie,
+					       screen, c, e)))
+	{
+	  verbose("X11 request failed.\n");
+	  EXCEPTION_RAISE(e, &x11_request_failed);
+	}
+      else
+	{
+	  
+	  return;
+	}
+    }
+  else
+    {
+      werror("Invalid x11 request.\n");
+      PROTOCOL_ERROR(e, "Invalid x11 request.");
+    }
+}
+
+struct channel_request
+x11_req_handler =
+{ STATIC_HEADER, do_x11_req };
+
+#endif /* WITH_X11_FORWARD */
