@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: gnu_stuff.c,v 1.4 2002/03/25 23:06:53 _cvs_pont Exp $
+ * @(#) $Id: gnu_stuff.c,v 1.7 2002/06/25 11:19:48 nisse Exp $
  *
  * gnu_stuff.c
  */
@@ -27,6 +27,7 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <errno.h>
 
 void lsftp_welcome()
 {
@@ -76,6 +77,8 @@ void help_option()
   printf( "\n");
   printf( "$LSFTP_RSH $LSFTP_BEFORE_ARGS <anything on the command line> $LSFTP_AFTER_ARGS\n");
   printf( "\n");
+  printf( "Report bugs to <bug-lsh@gnu.org>.\n" );
+  printf( "\n");
 
   exit(0); /* Exit successfully */
 }
@@ -113,3 +116,39 @@ void do_gnu_stuff( const char** argv )
 }
 
 
+#ifndef HAVE_CANONICALIZE_FILE_NAME
+
+/* Compability function, taken from a parted patch */
+
+char*
+canonicalize_file_name (const char* name)
+{
+  char* buf;
+  int size;
+  char* result;
+  
+#ifdef PATH_MAX
+  size = PATH_MAX;
+#else
+  /* Bigger is better; realpath has no way todo bounds checking.  */
+  size = 4096;
+#endif
+
+  /* Just in case realpath does not NULL terminate the string
+   * or it just fits in SIZE without a NULL terminator.  */
+  buf = calloc( size + 1, 1 );
+ 
+  if( !buf ) 
+    {
+      errno = ENOMEM;
+      return NULL;
+    }
+
+  result = realpath(name, buf);
+
+  if (! result)
+    free (buf);
+
+  return result;
+}
+#endif /* !HAVE_CANONICALIZE_FILE_NAME */
