@@ -1,5 +1,5 @@
 /*
- * @(#) $Id: sftp_bind.h,v 1.5 2002/05/25 16:21:46 _cvs_pont Exp $
+ * @(#) $Id: sftp_bind.h,v 1.6 2002/06/27 15:58:48 _cvs_pont Exp $
  *
  * sftp_bind.h 
  *
@@ -140,8 +140,8 @@ int lsftp_do_stat(const char* file, struct stat* st);
 int lsftp_handle_stat(struct sftp_callback *s,
 		      const struct lsftp_callback *l);
 
-int lsftp_do_realpath(const char* file, const char **destptr );
-int lsftp_handle_realpath(struct sftp_callback *s,
+int lsftp_do_realpath(const char* file, char** destptr );
+int lsftp_handle_realpath(struct sftp_callback* s,
 			  const struct lsftp_callback *l);
 
 int lsftp_do_mv(const char *src, const char *dst, const char *command);
