@@ -1,6 +1,6 @@
 /* tcpforward.h
  *
- * $Id: tcpforward.h,v 1.15 2000/08/04 00:17:21 nisse Exp $
+ * $Id: tcpforward.h,v 1.17 2001/08/28 18:33:08 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -63,8 +63,6 @@
      (name remote_port)
      (super forwarded_port)
      (vars
-       ; Invoked with the peer address_info as argument 
-       (c object command_continuation)
        ; Invoked when a forwarded_tcpip request is received.
        ; Called with the struct address_info *peer as argument.
        (callback object command)))
@@ -74,15 +72,13 @@ struct remote_port *
 make_remote_port(struct address_info *listen,
 		 struct command *callback);
 
-struct ssh_channel *make_tcpip_channel(struct lsh_fd *socket, UINT32 initial_window);
-void tcpip_channel_start_io(struct ssh_channel *c);
-
 struct channel_open *
 make_channel_open_direct_tcpip(struct command *callback);
 
 extern struct channel_open channel_open_forwarded_tcpip;
 
-struct global_request *make_tcpip_forward_request(struct command *callback);
+struct global_request *
+make_tcpip_forward_request(struct command *callback);
 
 extern struct global_request tcpip_cancel_forward;
 
