@@ -2,7 +2,7 @@
  *
  * User-related functions on UN*X
  *
- * $Id: unix_user.c,v 1.52 2002/07/02 19:42:36 nisse Exp $
+ * $Id: unix_user.c,v 1.52.2.1 2003/09/20 07:30:23 nisse Exp $
  */
 
 /* lsh, an implementation of the ssh protocol
@@ -813,6 +813,14 @@ do_spawn(struct lsh_user *u,
 
       trace("do_spawn: child after chdir\n");
       
+      /* We want to be a process group leader */
+      if (setsid() < 0)
+	{
+	  werror("unix_user: setsid failed, already process group leader?\n"
+		 "   %e\n", errno);
+	  _exit(EXIT_FAILURE);
+	}
+
 #if WITH_PTY_SUPPORT
       if (info->pty)
 	{
